@@ -82,9 +82,11 @@ sub new
     # is_method flag.
     while (my ($name,$c) = each(%{$self->{subcomps}})) {
 	$c->assign_subcomponent_properties($self,$name,0);
+	Scalar::Util::weaken($c->{owner}) if CAN_WEAKEN;
     }
     while (my ($name,$c) = each(%{$self->{methods}})) {
 	$c->assign_subcomponent_properties($self,$name,1);
+	Scalar::Util::weaken($c->{owner}) if CAN_WEAKEN;
     }
 
     return $self;
@@ -231,6 +233,8 @@ sub attr_if_exists {
     my $value;
     if ($self->_locate_inherited('attr',$name,\$value)) {
 	return $value;
+    } else {
+	return undef;
     }
 }
 
