@@ -36,7 +36,7 @@ BEGIN
 	     description => "error thrown in eval of the code for a component" },
 
 	   'HTML::Mason::Exception::Compilation::IncompatibleCompiler' =>
-	   { isa => 'HTML::Mason::Exception',
+	   { isa => 'HTML::Mason::Exception::Compilation',
 	     alias => 'wrong_compiler_error',
 	     description => "a component was compiled by a compiler/lexer with incompatible options.  recompilation is needed" },
 
@@ -172,8 +172,10 @@ sub filtered_frames
 	qw[
 	   (eval)
 	   Exception::Class::Base::throw
+	   Exception::Class::__ANON__
 	   HTML::Mason::Commands::__ANON__
 	   HTML::Mason::Component::run
+	   HTML::Mason::Exception::throw
 	   HTML::Mason::Exceptions::__ANON__
 	   HTML::Mason::Request::_run_comp
 	   ];
@@ -365,7 +367,7 @@ sub as_html
 %    foreach my $frame (@{$info->{frames}}) {
 %        my $f = $frame->filename; HTML::Mason::Escapes::basic_html_escape(\$f);
 %        my $l = $frame->line; HTML::Mason::Escapes::basic_html_escape(\$l);
-	<% $f %>:<% $l %><br>g
+	<% $f %>:<% $l %><br>
 %    }
   </td>
  </tr>
